@@ -13,7 +13,7 @@ use futures::{
     channel::oneshot::{self, Sender},
     future::Either,
 };
-use hyper::{Body, Request, Response};
+use hyper::{Request, Response};
 use libc::{EINVAL, EIO};
 use tokio::{
     runtime::{self, Runtime},
@@ -21,8 +21,8 @@ use tokio::{
 };
 
 use crate::{
-    auth::BasicAuthorization, exports::RawContextWrapper, ClientHandlerResult, DeviceHandlerResult,
-    Error, ProxyBuilder, ProxyHandle, RequestHandler,
+    auth::BasicAuthorization, exports::RawContextWrapper, Body, ClientHandlerResult,
+    DeviceHandlerResult, Error, ProxyBuilder, ProxyHandle, RequestHandler,
 };
 
 /// Foreign callback for asynchronous proxy construction.
@@ -85,6 +85,7 @@ impl RequestHandler for RawRequestHandler {
 
         tokio::task::spawn_blocking(move || {
             // we need to capture the whole request handler
+            #[allow(clippy::redundant_locals)]
             let this = this;
 
             let authorization = Box::into_raw(Box::new(authorization));
@@ -108,6 +109,7 @@ impl RequestHandler for RawRequestHandler {
 
         tokio::task::spawn_blocking(move || {
             // we need to capture the whole request handler
+            #[allow(clippy::redundant_locals)]
             let this = this;
 
             let request = Box::into_raw(Box::new(request));
